@@ -69,7 +69,7 @@ def parse_cmdlineoptions(arglist):
 		  "include-special-files", "include-symbolic-links",
 		  "list-at-time=", "list-changed-since=", "list-increments",
 		  "list-increment-sizes", "never-drop-acls", "no-acls",
-		  "no-compare-inode", "no-compression",
+		  "no-carbonfile", "no-compare-inode", "no-compression",
 		  "no-compression-regexp=", "no-eas", "no-file-statistics",
 		  "no-hard-links", "null-separator",
 		  "override-chars-to-quote=", "parsable-output",
@@ -143,6 +143,7 @@ def parse_cmdlineoptions(arglist):
 		elif opt == '--list-increment-sizes': action = 'list-increment-sizes'
 		elif opt == "--never-drop-acls": Globals.set("never_drop_acls", 1)
 		elif opt == "--no-acls": Globals.set("acls_active", 0)
+		elif opt == "--no-carbonfile": Globals.set("carbonfile_active", 0)
 		elif opt == "--no-compare-inode": Globals.set("compare_inode", 0)
 		elif opt == "--no-compression": Globals.set("compression", None)
 		elif opt == "--no-compression-regexp":
@@ -452,8 +453,10 @@ def backup_touch_curmirror_local(rpin, rpout):
 	"""
 	mirrorrp = Globals.rbdir.append("current_mirror.%s.%s" % (Time.curtimestr,
 															  "data"))
-	Log("Touching mirror marker %s" % mirrorrp.path, 6)
-	mirrorrp.touch()
+	Log("Writing mirror marker %s" % mirrorrp.path, 6)
+	try: pid = os.getpid()
+	except: pid = "NA"
+	mirrorrp.write_string("PID %s\n" % (pid,))
 	mirrorrp.fsync_with_dir()
 
 def backup_remove_curmirror_local():
@@ -603,7 +606,7 @@ Try restoring from an increment file (the filenames look like
 					   % (Globals.rbdir.path,))
 	elif result == 1: Log.FatalError(
 		"Previous backup to %s seems to have failed.\nRerun rdiff-backup "
-		"rdiff-with --check-destination-dir option to revert directory "
+		"with --check-destination-dir option to revert directory "
 		"to state before unsuccessful session." % (mirror_root.path,))
 
 def restore_set_root(rpin):
