@@ -283,6 +283,11 @@ class RORPath:
 		self.data = {'type': None}
 		self.file = None
 
+	def make_zero_dir(self, dir_rp):
+		"""Set self.data the same as dir_rp.data but with safe permissions"""
+		self.data = dir_rp.data.copy()
+		self.data['perms'] = 0700
+
 	def __nonzero__(self): return 1
 
 	def __eq__(self, other):
@@ -777,7 +782,12 @@ class RPath(RORPath):
 
 	def chown(self, uid, gid):
 		"""Set file's uid and gid"""
-		self.conn.C.lchown(self.path, uid, gid)
+		if self.issym():
+			try: self.conn.C.lchown(self.path, uid, gid)
+			except AttributeError:
+				log.Log("Warning: lchown missing, cannot change ownership "
+						"of symlink " + self.path, 2)
+		else: os.chown(self.path, uid, gid)
 		self.data['uid'] = uid
 		self.data['gid'] = gid
 
@@ -1040,8 +1050,10 @@ class RPath(RORPath):
 			mode = stat.S_IFBLK | 0600
 		else: raise RPathException
 		try: self.conn.os.mknod(self.path, mode, self.conn.os.makedev(major, minor))
-		except OSError, e:
-			if e.errno == errno.EPERM:
+		except (OSError, AttributeError), e:
+			if isinstance(e, AttributeError) or e.errno == errno.EPERM:
+				# AttributeError will be raised by Python 2.2, which
+				# doesn't have os.mknod
 				log.Log("unable to mknod %s -- using touch instead" % self.path, 4)
 				self.touch()
 		self.setdata()
